import types
from ..check.mixin import _check_attr_method_map
from ..error.check_error import CheckError


class check_dec(object):

    def __init__(self, f, fields, *args, **kwargs):
        self.func = f
        self.fields = fields

    def __call__(self, *args, **kwargs):
        res = self.func(*args, **kwargs)
        return res

    def __get__(self, instance, cls):
        return types.MethodType(self, instance, cls)


class check(object):
    """
    Decorator aimed to map attributes with check methods
    that will then validate the values of those attributes.

    @param fields: List of attribute names the defined method will check
    @type  fields: list
    @return: Newly defined method
    @rtype : function
    """

    def __init__(self, fields=[]):
        if not fields or not all(map(lambda x: isinstance(x, str),
                                     fields)):
            msg = ("Check decorator should receive a list of " +
                   "attributes to check.\nGot %s instead" % fields)
            raise CheckError(message=msg)
        self.fields = fields

    def __call__(self, func, *args, **kwargs):
        ins = check_dec(func, self.fields, *args, **kwargs)
        for f in self.fields:
            _check_attr_method_map[f] = [ins, ins.func.__code__.co_varnames]
        return ins



class WSError(BaseException):

    def __init__(self, ws_url, message=False, method=False,
                 attr=False, code=False, ws_method=False):
        self.ws_url = ws_url
        self.message = message
        self.method = method
        self.attr = attr
        self.code = code
        self.ws_method = ws_method

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        m = "\nWebService Error: (connected to `%s`)" % self.ws_url
        if self.message:
            if self.code:
                m += "\nResponse Code %s." % self.code
            m += "\n%s" % self.message
            return m
        if self.method:
            m += "\nThe function you are looking for " + \
                 "was not found in the Web Service."
            m += "\n(Function: `%s`)" % self.method
        if self.attr:
            m += "\nThe Attribute you are trying to get " + \
                 "was never set."
            if self.ws_method:
                m += "\n(Attribute: `%s`, WS Function: `%s`)" % \
                    (self.attr, self.ws_method)
            else:
                m += "\n(Attribute: `%s`)" % self.attr
        if self.code:
            m += "\nThe WebService answered with code %s." % self.code
        return m



class CheckError(BaseException):

    def __init__(self, message=False, method=False, attr=False,
                 not_found=False, value=False):
        self.method = method
        self.attr = attr
        self.message = message
        self.not_found = not_found
        self.value = value

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        m = False
        if self.message:
            m = "\n%s" % self.message
            return m
        if self.method and self.attr:
            m = "\nThe Attribute `%s` Failed the check on method `%s`." % \
                (self.attr, self.method)
            if self.value:
                m += "\nIt's value was `%s`" % self.value
        if self.attr and not self.method:
            if self.not_found:
                m = "\nCould not find a Check method for Attribute `%s`." % \
                    self.attr
            else:
                m = "\nThe Attribute `%s` Failed a Check." % self.attr
        if self.method and not self.attr:
            m = "\nThe Method `%s` could not validate an Attribute." % \
                self.method
        if not m:
            m = "\nThere was an Attribute that could not pass a Check."
        return m
